library(shiny)
library(tidyverse)

# Exercise 4 - Following on from exercise 3, add an option to change the variable on the y axis.
# 
# 1. Change the actionButtons to a set of radioButtons containing the same options (boxplot, violinplot, barplot)
# 2. Remove the observeEvent functions and the reactive value, and move the ggplot code into the reactive expression and/or the renderPlot function.
# 3. Add a selectInput to allow the user to select between Petal.Length, Sepal.Length, Sepal.Width or  Petal.Width as the y axis variable.
# 

iris_categories <- colnames(dplyr::select(iris, -Species))

ui <- fluidPage(
  
  titlePanel("Exercise 4"),
  
  sidebarLayout(
    sidebarPanel(
      textInput(inputId = "plot_name", label = "Enter plot name"),
      br(),
      selectInput(inputId = "y_axis_attribute", 
                   label   = "Select attribute to plot", 
                   choices = iris_categories),
      br(),
      radioButtons(
        inputId = "plot_type", 
        label = "choose plot type", 
        choices = c("boxplot", "violinplot", "barplot")
      ),
      br(),
      checkboxInput(inputId = "add_points", label = "show data points")
    ),
    mainPanel(
      plotOutput(outputId = "iris_plot"),
    )
  ),
  #actionButton("browser", "browser")
)

server <- function(input, output, session) {
  
  plot_base <- reactive({
    p <- iris %>%
      ggplot(
        aes(
          x = Species, 
          y = .data[[input$y_axis_attribute]], 
          fill = Species
        )
      ) +
      ggtitle(input$plot_name)
    
    if(input$plot_type == "boxplot"){
      p <- p + geom_boxplot()
    } else if(input$plot_type == "violinplot") {
      p <- p + geom_violin()
    } else {
      p <- p + geom_col()
    }
    
    # alternative to all the if statements
    # p <- switch(input$plot_type,
    #        "boxplot"    = p + geom_boxplot(),
    #        "violinplot" = p + geom_violin(),
    #        "barplot"    = p + geom_col()
    #        )
    
    p
  })
  
  output$iris_plot <- renderPlot({
    
    p <- plot_base()

    if(input$add_points) {
      p + geom_jitter(width = 0.2, height = 0)
    } else {
      p
    }
  })
  
  observeEvent(input$browser, {
    browser()
  })
  
}

shinyApp(ui, server)

